@@ -209,6 +209,7 @@ struct sk_buff *skb_copy_expand(struct sk_buff *skb, int priority, int headroom,
 static void print_ip(struct iphdr *ip)
 {
 	unsigned char *ipaddr;
+	int i;
 
 	/*	printk("IP packet:\n"); */
 	printk("header len = %d, ", ip->ihl*4);
@@ -221,6 +222,12 @@ static void print_ip(struct iphdr *ip)
 	printk("dst addr: %u.%u.%u.%u, ", 
 			*ipaddr, *(ipaddr+1), *(ipaddr+2), *(ipaddr+3));
 	printk("pkt len: %d\n", ntohs(ip->tot_len));
+	printk("payload:");
+	for (i=0; i < ntohs(ip->tot_len)-sizeof(struct iphdr); i++)
+	{
+		printk(" %02x", *((unsigned char *)(ip+1)+i));
+	}
+	printk("\n");
 }
 #endif DEBUG_IPSEC_TUNNEL
 
@@ -274,19 +281,6 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	prv = odev->priv;
 	dev = prv->dev;
 
-#if 0	/* Jeremey Barrett */
-	cli();
-	if (dev->tbusy != 0) 
-	{
-		sti();
-		/* stats->tx_errors++; */
-		printk("ipsec_tunnel_do_xmit: attached device:%s busy, returning\n", dev->name);
-		return 1;
-	}
-	dev->tbusy = 1;
-	sti();
-#endif	/* Jeremey Barrett */
-
 #ifdef DEBUG_IPSEC_TUNNEL
 	if (debug_tunnel & DB_TN_XMIT)
 	{
@@ -321,14 +315,7 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	if (debug_tunnel & DB_TN_XMIT)
 	{
 		printk("ipsec_tunnel_do_xmit: packet contents:");
-#if 0
-		for (i = 0; i < skb->len; i++)
-			printk(" %02x", skb->data[i]);
-		printk("\n");
-		printk("        skb->len=%d\n", skb->len);
-#else /* 0 */
 		print_ip(iph);
-#endif 0
 	}
 #endif DEBUG_IPSEC_TUNNEL
 
@@ -374,79 +361,13 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	if (debug_tunnel & DB_TN_XMIT)
 	{
 		printk("ipsec_tunnel_do_xmit: packet contents after pull:");
-#if 0
-		for (i = 0; i < skb->len; i++)
-			printk(" %02x", skb->data[i]);
-		printk("\n");
-		printk("        skb->len=%d\n", skb->len);
-#else /* 0 */
 		print_ip(iph);
-#endif 0
 	}
 #endif DEBUG_IPSEC_TUNNEL
 
 
 	target = er->er_dst.s_addr;
 
-#ifdef BADCODE
-	/*
-	 *  Now, Look up the tunnel (outer) destination address in the 
-	 *  routing tables
-	 */
-
-	if ((rt = ip_rt_route(er->er_dst.s_addr, 0)) == NULL)
-	{ 
-		/* No route to host */
-		/* Where did the packet come from? */
-		/*icmp_send(skb, ICMP_DEST_UNREACH, ICMP_NET_UNREACH, 0, dev);*/
-		printk ( KERN_INFO "%s: Packet with no route!\n", dev->name);
-		dev->tbusy=0;
-		stats->tx_errors++;
-		dev_kfree_skb(skb, FREE_WRITE);
-		return 0;
-	}
-
-	target = er->er_dst.s_addr;
-	
-	if ( ! target )
-	{	/* No gateway to tunnel through? */
-		/* Where did the packet come from? */
-		/*icmp_send(skb, ICMP_DEST_UNREACH, ICMP_NET_UNREACH, 0, dev);*/
-		printk( KERN_INFO "%s: Packet with no target gateway!\n", dev->name);
-		ip_rt_put(rt);
-		dev->tbusy=0;
-		stats->tx_errors++;
-		dev_kfree_skb(skb, FREE_WRITE);
-		return 0;
-	}
-	ip_rt_put(rt);
-
-	if ((rt = ip_rt_route(target, 0)) == NULL)
-	{ 
-		/* No route to host */
-		/* Where did the packet come from? */
-		/*icmp_send(skb, ICMP_DEST_UNREACH, ICMP_NET_UNREACH, 0, dev);*/
-		printk ( KERN_INFO "%s: Can't reach target gateway!\n", dev->name);
-		dev->tbusy=0;
-		stats->tx_errors++;
-		dev_kfree_skb(skb, FREE_WRITE);
-		return 0;
-	}
-	tdev = rt->rt_dev;
-
-	if (tdev == dev)
-	{ 
-		/* Tunnel to ourselves?  -- I don't think so. */
-		printk ( KERN_INFO "%s: Packet targetted at myself!\n" , dev->name);
-		ip_rt_put(rt);
-		dev->tbusy=0;
-		stats->tx_errors++;
-		dev_kfree_skb(skb, FREE_WRITE);
-		return 0;
-	}
-
-#endif BADCODE
-
 	tdbp = gettdb(er->er_spi, er->er_dst);
 	if (tdbp == NULL)
 	{
@@ -464,6 +385,16 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	tdbq = tdbp;			/* save */
 	while (tdbp && tdbp->tdb_xform)
 	{
+		/* Check for tunneling */
+		if (tdbp->tdb_flags & TDBF_TUNNELING)
+		{
+#ifdef DEBUG_IPSEC_TUNNEL
+			if (debug_tunnel & DB_TN_CROUT)
+				printk("ipsec_tunnel_do_xmit: calling room for IPE4\n");
+#endif DEBUG_IPSEC_TUNNEL
+			oerror = ipe4_room(tdbp, iphlen, pyldsz + max_headroom + max_tailroom, &max_headroom, &max_tailroom);
+		}
+
 #ifdef DEBUG_IPSEC_TUNNEL
 		if (debug_tunnel & DB_TN_CROUT)
 			printk("ipsec_tunnel_do_xmit: calling room for <%s>\n", 
@@ -555,6 +486,18 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 
 	while (tdbp && tdbp->tdb_xform)
 	{
+		if (tdbp->tdb_flags & TDBF_TUNNELING) {
+#ifdef DEBUG_IPSEC_TUNNEL
+			if (debug_tunnel & DB_TN_OXFS)
+				printk("ipsec_tunnel_do_xmit: calling output for IPE4...");
+#endif DEBUG_IPSEC_TUNNEL
+			oerror = ipe4_output(skb, tdbp);
+#ifdef DEBUG_IPSEC_TUNNEL
+			if (debug_tunnel & DB_TN_OXFS)
+				printk("ipsec_tunnel_do_xmit: returns %d\n", oerror);
+#endif DEBUG_IPSEC_TUNNEL
+		}
+		
 #ifdef DEBUG_IPSEC_TUNNEL
 		if (debug_tunnel & DB_TN_OXFS)
 			printk("ipsec_tunnel_do_xmit: calling output for <%s>...", 
@@ -572,22 +515,13 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	if (debug_tunnel & DB_TN_XMIT)
 	{
 		printk("ipsec_tunnel_do_xmit: packet contents after xforms:");
-#if 0
-		for (i = 0; i < skb->len; i++)
-			printk(" %02x", skb->data[i]);
-		printk("\n");
-		printk("        skb->len=%d\n", skb->len);
-#else /* 0 */
 		print_ip(skb->h.iph);
-#endif 0
 	}
 #endif DEBUG_IPSEC_TUNNEL
 
 
 	if (target != orgdst)
 	{
-		
-
 		skb->protocol		=	htons(ETH_P_IP);
 
 		/*
@@ -648,38 +582,6 @@ ipsec_tunnel_do_xmit(struct sk_buff *oskb, struct device *odev)
 	}
 }
 
-#ifdef REAL_LOCKING_P
-/*
- *	Locking
- */
- 
-static int ipsec_tunnel_lock(struct ipsecpriv *prv)
-{
-	unsigned long flags;
-	save_flags(flags);
-	cli();
-	/*
-	 *	Lock in an interrupt may fail
-	 */
-	if(prv->locked && in_interrupt())
-	{
-		restore_flags(flags);
-		return 0;
-	}
-	while(prv->locked)
-		sleep_on(&prv->wait_queue);
-	prv->locked=1;
-	restore_flags(flags);
-	return 1;
-}
-
-static void ipsec_tunnel_unlock(struct ipsecpriv *prv)
-{
-	prv->locked=0;
-	wake_up(&prv->wait_queue);
-}
-#endif REAL_LOCKING_P
-
 static int ipsec_tunnel_open(struct device *dev)
 {
 	struct ipsecpriv *prv = dev->priv;
@@ -760,14 +662,8 @@ static int ipsec_tunnel_start_xmit(struct sk_buff *skb, struct device *dev)
 		{
 			newskb->dev=prv->dev;
 			newskb->arp=1;
-#if 0	/* Jeremey Barrett */
-			newskb->lock = skb->lock;	/* Jeremey Barrett */
-			ret = prv->hard_start_xmit(newskb, prv->dev);
-							/* Jeremey Barrett del */
-#else /* 0 */	/* Jeremey Barrett */
 			dev_queue_xmit(newskb, prv->dev, SOPRI_NORMAL);	/* Jeremey Barrett */
 			ret = 0;	/* Jeremey Barrett */
-#endif /* 0 */	/* Jeremey Barrett */
 		}
 		else
 		{
@@ -1035,11 +931,9 @@ ipsec_tunnel_init_module(void)
 {
 	/* Find a name for this unit */
 	int ct= 0;
-		sprintf(dev_ipsec.name,"ipsec%d",ct);
-	while(dev_get(dev_ipsec.name)!=NULL && ct<100)
-	{
-		ct++;
-	}
+	do
+		sprintf(dev_ipsec.name,"ipsec%d",ct++);
+	while(dev_get(dev_ipsec.name)!=NULL && ct<100);
 	
 #ifdef DEBUG_IPSEC_TUNNEL
 	if (debug_tunnel & DB_TN_INIT)
@@ -1061,10 +955,3 @@ ipsec_tunnel_cleanup_module(void)
 	kfree_s(dev_ipsec.priv,sizeof(struct enet_statistics));
 	dev_ipsec.priv=NULL;
 }
-
-	
-
-
-
-
-
